package oauth_provider

import (
	"context"
	"git.noahlan.cn/n-admin/n-admin-server/rpc/core/internal/utils/hander"
	"git.noahlan.cn/noahlan/ntool-biz/core/nstatus/code"
	"git.noahlan.cn/noahlan/ntool-biz/core/nstatus/msg"

	"git.noahlan.cn/n-admin/n-admin-server/rpc/core/internal/svc"
	"git.noahlan.cn/n-admin/n-admin-server/rpc/core/types/core"
)

type CreateOauthProviderLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewCreateOauthProviderLogic(ctx context.Context, svcCtx *svc.ServiceContext) *CreateOauthProviderLogic {
	return &CreateOauthProviderLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

// OauthProvider management
func (l *CreateOauthProviderLogic) CreateOauthProvider(in *core.OauthProviderInfo) (*core.BaseIDResp, error) {
	create := l.svcCtx.DB.OauthProvider.Create().
		SetName(in.Name).
		SetClientID(in.ClientId).
		SetClientSecret(in.ClientSecret).
		SetRedirectURL(in.RedirectUrl).
		SetScopes(in.Scopes).
		SetAuthURL(in.AuthUrl).
		SetTokenURL(in.TokenUrl).
		SetInfoURL(in.InfoUrl).
		SetDescription(in.Description).
		SetSystem(in.System)
	if in.ClientId != "" && in.ClientSecret != "" {
		create.SetInit(true)
	}
	result, err := create.Save(l.ctx)
	if err != nil {
		return nil, hander.HandleEntErr(err, in)
	}
	return &core.BaseIDResp{
		ID:   result.ID,
		Code: code.StatusOK,
		Msg:  msg.CreateSuccess,
	}, nil
}
